#ifndef CPM_CONSTANTS_H_
#define CPM_CONSTANTS_H_

#ifdef __cplusplus
extern "C" {
#endif

/*! Types of MPI collective communication operation */
typedef enum {
	/*! Scatter/Gather */
	SG,
	/*! Scatterv/Gatherv */
	SGV,
	/*! Broadcast/Reduce */
	BR
} CPM_coll_op_types;

/*! Implemented algorithms of MPI collective communication operations */
typedef enum {
P2P=0,
//broadcast operations
MPIB_BCAST_BINOMIAL, 
HOCKNEY_BCAST_DFS_BINOMIAL_MIN, 
HOCKNEY_BCAST_DFS_BINOMIAL_MAX, 
HOCKNEY_BCAST_BFS_BINOMIAL_MIN,
HOCKNEY_BCAST_BFS_BINOMIAL_MAX, 
HOCKNEY_BCAST_UCS_BINOMIAL_MIN,
HOCKNEY_BCAST_UCS_BINOMIAL_MAX,
LININTERP_BCAST_DFS_BINOMIAL_MIN, 
LININTERP_BCAST_DFS_BINOMIAL_MAX, 
LININTERP_BCAST_BFS_BINOMIAL_MIN,
LININTERP_BCAST_BFS_BINOMIAL_MAX, 
LININTERP_BCAST_UCS_BINOMIAL_MIN,
LININTERP_BCAST_UCS_BINOMIAL_MAX,
PLOGP_BCAST_DFS_BINOMIAL_MIN,
PLOGP_BCAST_DFS_BINOMIAL_MAX,
PLOGP_BCAST_BFS_BINOMIAL_MIN,
PLOGP_BCAST_BFS_BINOMIAL_MAX,
PLOGP_BCAST_UCS_BINOMIAL_MIN,
PLOGP_BCAST_UCS_BINOMIAL_MAX,
LOGGP_BCAST_DFS_BINOMIAL_MIN,
LOGGP_BCAST_DFS_BINOMIAL_MAX,
LOGGP_BCAST_BFS_BINOMIAL_MIN,
LOGGP_BCAST_BFS_BINOMIAL_MAX,
LOGGP_BCAST_UCS_BINOMIAL_MIN,
LOGGP_BCAST_UCS_BINOMIAL_MAX,
//reduce operations
MPIB_REDUCE_BINOMIAL, 
HOCKNEY_REDUCE_DFS_BINOMIAL_MIN, 
HOCKNEY_REDUCE_DFS_BINOMIAL_MAX, 
HOCKNEY_REDUCE_BFS_BINOMIAL_MIN,
HOCKNEY_REDUCE_BFS_BINOMIAL_MAX, 
HOCKNEY_REDUCE_UCS_BINOMIAL_MIN,
HOCKNEY_REDUCE_UCS_BINOMIAL_MAX,
LININTERP_REDUCE_DFS_BINOMIAL_MIN, 
LININTERP_REDUCE_DFS_BINOMIAL_MAX, 
LININTERP_REDUCE_BFS_BINOMIAL_MIN,
LININTERP_REDUCE_BFS_BINOMIAL_MAX, 
LININTERP_REDUCE_UCS_BINOMIAL_MIN,
LININTERP_REDUCE_UCS_BINOMIAL_MAX,
PLOGP_REDUCE_DFS_BINOMIAL_MIN,
PLOGP_REDUCE_DFS_BINOMIAL_MAX,
PLOGP_REDUCE_BFS_BINOMIAL_MIN,
PLOGP_REDUCE_BFS_BINOMIAL_MAX,
PLOGP_REDUCE_UCS_BINOMIAL_MIN,
PLOGP_REDUCE_UCS_BINOMIAL_MAX,
LOGGP_REDUCE_DFS_BINOMIAL_MIN,
LOGGP_REDUCE_DFS_BINOMIAL_MAX,
LOGGP_REDUCE_BFS_BINOMIAL_MIN,
LOGGP_REDUCE_BFS_BINOMIAL_MAX,
LOGGP_REDUCE_UCS_BINOMIAL_MIN,
LOGGP_REDUCE_UCS_BINOMIAL_MAX,
//scatter operations
MPIB_SCATTER_LINEAR,
LMO_SCATTER_LINEAR,
MPIB_SCATTER_BINOMIAL, 
HOCKNEY_SCATTER_DFS_BINOMIAL_MIN, 
HOCKNEY_SCATTER_DFS_BINOMIAL_MAX, 
HOCKNEY_SCATTER_BFS_BINOMIAL_MIN,
HOCKNEY_SCATTER_BFS_BINOMIAL_MAX, 
HOCKNEY_SCATTER_UCS_BINOMIAL_MIN,
HOCKNEY_SCATTER_UCS_BINOMIAL_MAX,
LININTERP_SCATTER_DFS_BINOMIAL_MIN, 
LININTERP_SCATTER_DFS_BINOMIAL_MAX, 
LININTERP_SCATTER_BFS_BINOMIAL_MIN,
LININTERP_SCATTER_BFS_BINOMIAL_MAX, 
LININTERP_SCATTER_UCS_BINOMIAL_MIN,
LININTERP_SCATTER_UCS_BINOMIAL_MAX,
PLOGP_SCATTER_DFS_BINOMIAL_MIN,
PLOGP_SCATTER_DFS_BINOMIAL_MAX,
PLOGP_SCATTER_BFS_BINOMIAL_MIN,
PLOGP_SCATTER_BFS_BINOMIAL_MAX,
PLOGP_SCATTER_UCS_BINOMIAL_MIN,
PLOGP_SCATTER_UCS_BINOMIAL_MAX,
LOGGP_SCATTER_DFS_BINOMIAL_MIN,
LOGGP_SCATTER_DFS_BINOMIAL_MAX,
LOGGP_SCATTER_BFS_BINOMIAL_MIN,
LOGGP_SCATTER_BFS_BINOMIAL_MAX,
LOGGP_SCATTER_UCS_BINOMIAL_MIN,
LOGGP_SCATTER_UCS_BINOMIAL_MAX,
//gather operations
MPIB_GATHER_LINEAR,
LMO_GATHER_LINEAR,
MPIB_GATHER_BINOMIAL, 
HOCKNEY_GATHER_DFS_BINOMIAL_MIN, 
HOCKNEY_GATHER_DFS_BINOMIAL_MAX, 
HOCKNEY_GATHER_BFS_BINOMIAL_MIN,
HOCKNEY_GATHER_BFS_BINOMIAL_MAX, 
HOCKNEY_GATHER_UCS_BINOMIAL_MIN,
HOCKNEY_GATHER_UCS_BINOMIAL_MAX,
LININTERP_GATHER_DFS_BINOMIAL_MIN, 
LININTERP_GATHER_DFS_BINOMIAL_MAX, 
LININTERP_GATHER_BFS_BINOMIAL_MIN,
LININTERP_GATHER_BFS_BINOMIAL_MAX, 
LININTERP_GATHER_UCS_BINOMIAL_MIN,
LININTERP_GATHER_UCS_BINOMIAL_MAX,
PLOGP_GATHER_DFS_BINOMIAL_MIN,
PLOGP_GATHER_DFS_BINOMIAL_MAX,
PLOGP_GATHER_BFS_BINOMIAL_MIN,
PLOGP_GATHER_BFS_BINOMIAL_MAX,
PLOGP_GATHER_UCS_BINOMIAL_MIN,
PLOGP_GATHER_UCS_BINOMIAL_MAX,
LOGGP_GATHER_DFS_BINOMIAL_MIN,
LOGGP_GATHER_DFS_BINOMIAL_MAX,
LOGGP_GATHER_BFS_BINOMIAL_MIN,
LOGGP_GATHER_BFS_BINOMIAL_MAX,
LOGGP_GATHER_UCS_BINOMIAL_MIN,
LOGGP_GATHER_UCS_BINOMIAL_MAX,
//scatterv operations
MPIB_SCATTERV_LINEAR,
MPIB_SCATTERV_BINOMIAL,
MPIB_SCATTERV_TRAFF,
HOCKNEY_SCATTERV_DFS_BINOMIAL_MIN, 
HOCKNEY_SCATTERV_DFS_BINOMIAL_MAX, 
HOCKNEY_SCATTERV_BFS_BINOMIAL_MIN,
HOCKNEY_SCATTERV_BFS_BINOMIAL_MAX, 
HOCKNEY_SCATTERV_UCS_BINOMIAL_MIN,
HOCKNEY_SCATTERV_UCS_BINOMIAL_MAX,
HOCKNEY_SCATTERV_TRAFF,
LININTERP_SCATTERV_DFS_BINOMIAL_MIN, 
LININTERP_SCATTERV_DFS_BINOMIAL_MAX, 
LININTERP_SCATTERV_BFS_BINOMIAL_MIN,
LININTERP_SCATTERV_BFS_BINOMIAL_MAX, 
LININTERP_SCATTERV_UCS_BINOMIAL_MIN,
LININTERP_SCATTERV_UCS_BINOMIAL_MAX,
LININTERP_SCATTERV_TRAFF,
PLOGP_SCATTERV_DFS_BINOMIAL_MIN,
PLOGP_SCATTERV_DFS_BINOMIAL_MAX,
PLOGP_SCATTERV_BFS_BINOMIAL_MIN,
PLOGP_SCATTERV_BFS_BINOMIAL_MAX,
PLOGP_SCATTERV_UCS_BINOMIAL_MIN,
PLOGP_SCATTERV_UCS_BINOMIAL_MAX,
PLOGP_SCATTERV_TRAFF,
LOGGP_SCATTERV_DFS_BINOMIAL_MIN,
LOGGP_SCATTERV_DFS_BINOMIAL_MAX,
LOGGP_SCATTERV_BFS_BINOMIAL_MIN,
LOGGP_SCATTERV_BFS_BINOMIAL_MAX,
LOGGP_SCATTERV_UCS_BINOMIAL_MIN,
LOGGP_SCATTERV_UCS_BINOMIAL_MAX,
LOGGP_SCATTERV_TRAFF,
//gatherv operations
MPIB_GATHERV_LINEAR,
MPIB_GATHERV_BINOMIAL,
MPIB_GATHERV_TRAFF,
HOCKNEY_GATHERV_DFS_BINOMIAL_MIN, 
HOCKNEY_GATHERV_DFS_BINOMIAL_MAX, 
HOCKNEY_GATHERV_BFS_BINOMIAL_MIN,
HOCKNEY_GATHERV_BFS_BINOMIAL_MAX, 
HOCKNEY_GATHERV_UCS_BINOMIAL_MIN,
HOCKNEY_GATHERV_UCS_BINOMIAL_MAX,
HOCKNEY_GATHERV_TRAFF,
LININTERP_GATHERV_DFS_BINOMIAL_MIN, 
LININTERP_GATHERV_DFS_BINOMIAL_MAX, 
LININTERP_GATHERV_BFS_BINOMIAL_MIN,
LININTERP_GATHERV_BFS_BINOMIAL_MAX, 
LININTERP_GATHERV_UCS_BINOMIAL_MIN,
LININTERP_GATHERV_UCS_BINOMIAL_MAX,
LININTERP_GATHERV_TRAFF,
PLOGP_GATHERV_DFS_BINOMIAL_MIN,
PLOGP_GATHERV_DFS_BINOMIAL_MAX,
PLOGP_GATHERV_BFS_BINOMIAL_MIN,
PLOGP_GATHERV_BFS_BINOMIAL_MAX,
PLOGP_GATHERV_UCS_BINOMIAL_MIN,
PLOGP_GATHERV_UCS_BINOMIAL_MAX,
PLOGP_GATHERV_TRAFF,
LOGGP_GATHERV_DFS_BINOMIAL_MIN,
LOGGP_GATHERV_DFS_BINOMIAL_MAX,
LOGGP_GATHERV_BFS_BINOMIAL_MIN,
LOGGP_GATHERV_BFS_BINOMIAL_MAX,
LOGGP_GATHERV_UCS_BINOMIAL_MIN,
LOGGP_GATHERV_UCS_BINOMIAL_MAX,
LOGGP_GATHERV_TRAFF,
//size 
NUMBER_OF_OPS 
} CPM_coll_ops; 

extern const char* CPM_coll_op_names[];

#ifdef __cplusplus
}
#endif

#endif //CPM_CONSTANTS_H
