#include "config.h"
#include "cpm_collectives.h"
#include "MPIBlib/collectives/sgv_flat.h"
#include <gsl/gsl_heapsort.h>

/*! Prediction sorter for generic sorted flat-tree scatterv/gatherv */
typedef struct CPM_SGv_sorter {
	MPIB_SGv_sorter base;
	MPIB_sort_order order;
	CPM_predictor* predictor;
	int root;
} CPM_SGv_sorter;

/*!
 * Sorts predictions. Result: indices (ranks).
 * We don't use the double gsl_sort_index because it provides only asc order.
 */
void CPM_SGv_sort_predictions(void* _this, const int size, const int* counts, size_t* indices) {
	CPM_predictor* predictor = ((CPM_SGv_sorter*)_this)->predictor;
	int root = ((CPM_SGv_sorter*)_this)->root;
	double* predictions = (double*)malloc(sizeof(double) * size);
	int i;
	for (i = 0; i < size; i++)
		predictions[i] = predictor->predict_p2p(predictor, root, i, counts[i]);
	int asc(const double* a, const double* b) {
		if (a > b)
			return 1;
		if (a < b)
			return -1;
		return 0;
	}
	int dsc(const double* a, const double* b) {
		if (a > b)
			return 1;
		if (a < b)
			return -1;
		return 0;
	}
	if (((CPM_SGv_sorter*)_this)->order == DESC)
		gsl_heapsort_index(indices, predictions, size, sizeof(double), (int (*)(const void*, const void*))dsc);
	else
		gsl_heapsort_index(indices, predictions, size, sizeof(double), (int (*)(const void*, const void*))asc);
	free(predictions);
}

int CPM_Scatterv_sorted_flat(CPM_predictor* predictor, MPIB_sort_order order,
	void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	CPM_SGv_sorter sorter = {{CPM_SGv_sort_predictions}, order, predictor, root};
	return MPIB_Scatterv_sorted_flat(&sorter.base, sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int CPM_Gatherv_sorted_flat(CPM_predictor* predictor, MPIB_sort_order order,
	void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	CPM_SGv_sorter sorter = {{CPM_SGv_sort_predictions}, order, predictor, root};
	return MPIB_Gatherv_sorted_flat(&sorter.base, sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
}
