#include "config.h"
#include "models/cpm_utils.h"
#include "cpm_collectives.h"
#include "cpm_functional.hpp"
#include "MPIBlib/collectives/sgv_tree_algorithms.hpp"
#include "models/cpm_sgv_tree_builders.hpp"

using namespace CPM::SGv;

extern "C" int CPM_Scatterv_Traff(CPM_predictor* predictor, void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	return MPIB_Scatterv_tree_algorithm(Traff_builder(predictor), R2L,
		sendbuf, sendcounts, displs, sendtype,
		recvbuf, recvcount, recvtype,
		root, comm);
}

extern "C" int CPM_Gatherv_Traff(CPM_predictor* predictor, void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	return MPIB_Gatherv_tree_algorithm(Traff_builder(predictor), L2R,
		sendbuf, sendcount, sendtype,
		recvbuf, recvcounts, displs, recvtype,
		root, comm);
}

extern "C" int CPM_Scatterv_dfs_binomial(CPM_predictor* predictor, CPM_next_node_strategy next_node,
	void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	return MPIB_Scatterv_tree_algorithm(DFS_binomial_builder(predictor, next_node), R2L,
		sendbuf, sendcounts, displs, sendtype,
		recvbuf, recvcount, recvtype,
		root, comm);
		
}

extern "C" int CPM_Gatherv_dfs_binomial(CPM_predictor* predictor, CPM_next_node_strategy next_node,
	void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	return MPIB_Gatherv_tree_algorithm(DFS_binomial_builder(predictor, next_node), L2R,
		sendbuf, sendcount, sendtype,
		recvbuf, recvcounts, displs, recvtype,
		root, comm);
}

extern "C" int CPM_Scatterv_bfs_binomial(CPM_predictor* predictor, CPM_next_node_strategy next_node,
	void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	return MPIB_Scatterv_tree_algorithm(BFS_binomial_builder(predictor, next_node), L2R,
		sendbuf, sendcounts, displs, sendtype,
		recvbuf, recvcount, recvtype,
		root, comm);
}

extern "C" int CPM_Gatherv_bfs_binomial(CPM_predictor* predictor, CPM_next_node_strategy next_node,
	void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int *recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	return MPIB_Gatherv_tree_algorithm(BFS_binomial_builder(predictor, next_node), R2L,
		sendbuf, sendcount, sendtype,
		recvbuf, recvcounts, displs, recvtype,
		root, comm);
}

extern "C" int CPM_Scatterv_ucs_binomial(CPM_predictor* predictor, CPM_next_node_strategy next_node,
	void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	return MPIB_Scatterv_tree_algorithm(UCS_binomial_builder(predictor, next_node), L2R,
		sendbuf, sendcounts, displs, sendtype,
		recvbuf, recvcount, recvtype,
		root, comm);
		
}

extern "C" int CPM_Gatherv_ucs_binomial(CPM_predictor* predictor, CPM_next_node_strategy next_node,
	void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int *recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	return MPIB_Gatherv_tree_algorithm(UCS_binomial_builder(predictor, next_node), R2L,
		sendbuf, sendcount, sendtype,
		recvbuf, recvcounts, displs, recvtype,
		root, comm);
		
}
