#include "MPIBlib/collectives/libmpib_coll.h"
#include "config.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "models/cpm.h"
#include "cpm_coll.h"

const char* CPM_coll_op_names[] = {
"P2P",
//broadcast operations
"MPIB_BCAST_BINOMIAL",
"HOCKNEY_BCAST_DFS_BINOMIAL_MIN",
"HOCKNEY_BCAST_DFS_BINOMIAL_MAX",
"HOCKNEY_BCAST_BFS_BINOMIAL_MIN",
"HOCKNEY_BCAST_BFS_BINOMIAL_MAX",
"HOCKNEY_BCAST_UCS_BINOMIAL_MIN",
"HOCKNEY_BCAST_UCS_BINOMIAL_MAX",
"LININTERP_BCAST_DFS_BINOMIAL_MIN",
"LININTERP_BCAST_DFS_BINOMIAL_MAX",
"LININTERP_BCAST_BFS_BINOMIAL_MIN",
"LININTERP_BCAST_BFS_BINOMIAL_MAX",
"LININTERP_BCAST_UCS_BINOMIAL_MIN",
"LININTERP_BCAST_UCS_BINOMIAL_MAX",
"PLOGP_BCAST_DFS_BINOMIAL_MIN",
"PLOGP_BCAST_DFS_BINOMIAL_MAX",
"PLOGP_BCAST_BFS_BINOMIAL_MIN",
"PLOGP_BCAST_BFS_BINOMIAL_MAX",
"PLOGP_BCAST_UCS_BINOMIAL_MIN",
"PLOGP_BCAST_UCS_BINOMIAL_MAX",
"LOGGP_BCAST_DFS_BINOMIAL_MIN",
"LOGGP_BCAST_DFS_BINOMIAL_MAX",
"LOGGP_BCAST_BFS_BINOMIAL_MIN",
"LOGGP_BCAST_BFS_BINOMIAL_MAX",
"LOGGP_BCAST_UCS_BINOMIAL_MIN",
"LOGGP_BCAST_UCS_BINOMIAL_MAX",
//reduce operations
"MPIB_REDUCE_BINOMIAL",
"HOCKNEY_REDUCE_DFS_BINOMIAL_MIN",
"HOCKNEY_REDUCE_DFS_BINOMIAL_MAX",
"HOCKNEY_REDUCE_BFS_BINOMIAL_MIN",
"HOCKNEY_REDUCE_BFS_BINOMIAL_MAX",
"HOCKNEY_REDUCE_UCS_BINOMIAL_MIN",
"HOCKNEY_REDUCE_UCS_BINOMIAL_MAX",
"LININTERP_REDUCE_DFS_BINOMIAL_MIN",
"LININTERP_REDUCE_DFS_BINOMIAL_MAX",
"LININTERP_REDUCE_BFS_BINOMIAL_MIN",
"LININTERP_REDUCE_BFS_BINOMIAL_MAX",
"LININTERP_REDUCE_UCS_BINOMIAL_MIN",
"LININTERP_REDUCE_UCS_BINOMIAL_MAX",
"PLOGP_REDUCE_DFS_BINOMIAL_MIN",
"PLOGP_REDUCE_DFS_BINOMIAL_MAX",
"PLOGP_REDUCE_BFS_BINOMIAL_MIN",
"PLOGP_REDUCE_BFS_BINOMIAL_MAX",
"PLOGP_REDUCE_UCS_BINOMIAL_MIN",
"PLOGP_REDUCE_UCS_BINOMIAL_MAX",
"LOGGP_REDUCE_DFS_BINOMIAL_MIN",
"LOGGP_REDUCE_DFS_BINOMIAL_MAX",
"LOGGP_REDUCE_BFS_BINOMIAL_MIN",
"LOGGP_REDUCE_BFS_BINOMIAL_MAX",
"LOGGP_REDUCE_UCS_BINOMIAL_MIN",
"LOGGP_REDUCE_UCS_BINOMIAL_MAX",
//scatter operations
"MPIB_SCATTER_LINEAR",
"LMO_SCATTER_LINEAR",
"MPIB_SCATTER_BINOMIAL",
"HOCKNEY_SCATTER_DFS_BINOMIAL_MIN",
"HOCKNEY_SCATTER_DFS_BINOMIAL_MAX",
"HOCKNEY_SCATTER_BFS_BINOMIAL_MIN",
"HOCKNEY_SCATTER_BFS_BINOMIAL_MAX",
"HOCKNEY_SCATTER_UCS_BINOMIAL_MIN",
"HOCKNEY_SCATTER_UCS_BINOMIAL_MAX",
"LININTERP_SCATTER_DFS_BINOMIAL_MIN",
"LININTERP_SCATTER_DFS_BINOMIAL_MAX",
"LININTERP_SCATTER_BFS_BINOMIAL_MIN",
"LININTERP_SCATTER_BFS_BINOMIAL_MAX",
"LININTERP_SCATTER_UCS_BINOMIAL_MIN",
"LININTERP_SCATTER_UCS_BINOMIAL_MAX",
"PLOGP_SCATTER_DFS_BINOMIAL_MIN",
"PLOGP_SCATTER_DFS_BINOMIAL_MAX",
"PLOGP_SCATTER_BFS_BINOMIAL_MIN",
"PLOGP_SCATTER_BFS_BINOMIAL_MAX",
"PLOGP_SCATTER_UCS_BINOMIAL_MIN",
"PLOGP_SCATTER_UCS_BINOMIAL_MAX",
"LOGGP_SCATTER_DFS_BINOMIAL_MIN",
"LOGGP_SCATTER_DFS_BINOMIAL_MAX",
"LOGGP_SCATTER_BFS_BINOMIAL_MIN",
"LOGGP_SCATTER_BFS_BINOMIAL_MAX",
"LOGGP_SCATTER_UCS_BINOMIAL_MIN",
"LOGGP_SCATTER_UCS_BINOMIAL_MAX",
//gather operations
"MPIB_GATHER_LINEAR",
"LMO_GATHER_LINEAR",
"MPIB_GATHER_BINOMIAL",
"HOCKNEY_GATHER_DFS_BINOMIAL_MIN",
"HOCKNEY_GATHER_DFS_BINOMIAL_MAX",
"HOCKNEY_GATHER_BFS_BINOMIAL_MIN",
"HOCKNEY_GATHER_BFS_BINOMIAL_MAX",
"HOCKNEY_GATHER_UCS_BINOMIAL_MIN",
"HOCKNEY_GATHER_UCS_BINOMIAL_MAX",
"LININTERP_GATHER_DFS_BINOMIAL_MIN",
"LININTERP_GATHER_DFS_BINOMIAL_MAX",
"LININTERP_GATHER_BFS_BINOMIAL_MIN",
"LININTERP_GATHER_BFS_BINOMIAL_MAX",
"LININTERP_GATHER_UCS_BINOMIAL_MIN",
"LININTERP_GATHER_UCS_BINOMIAL_MAX",
"PLOGP_GATHER_DFS_BINOMIAL_MIN",
"PLOGP_GATHER_DFS_BINOMIAL_MAX",
"PLOGP_GATHER_BFS_BINOMIAL_MIN",
"PLOGP_GATHER_BFS_BINOMIAL_MAX",
"PLOGP_GATHER_UCS_BINOMIAL_MIN",
"PLOGP_GATHER_UCS_BINOMIAL_MAX",
"LOGGP_GATHER_DFS_BINOMIAL_MIN",
"LOGGP_GATHER_DFS_BINOMIAL_MAX",
"LOGGP_GATHER_BFS_BINOMIAL_MIN",
"LOGGP_GATHER_BFS_BINOMIAL_MAX",
"LOGGP_GATHER_UCS_BINOMIAL_MIN",
"LOGGP_GATHER_UCS_BINOMIAL_MAX",
//scatterv operations
"MPIB_SCATTERV_LINEAR",
"MPIB_SCATTERV_BINOMIAL",
"MPIB_SCATTERV_TRAFF",
"HOCKNEY_SCATTERV_DFS_BINOMIAL_MIN", 
"HOCKNEY_SCATTERV_DFS_BINOMIAL_MAX", 
"HOCKNEY_SCATTERV_BFS_BINOMIAL_MIN",
"HOCKNEY_SCATTERV_BFS_BINOMIAL_MAX", 
"HOCKNEY_SCATTERV_UCS_BINOMIAL_MIN",
"HOCKNEY_SCATTERV_UCS_BINOMIAL_MAX",
"HOCKNEY_SCATTERV_TRAFF",
"LININTERP_SCATTERV_DFS_BINOMIAL_MIN", 
"LININTERP_SCATTERV_DFS_BINOMIAL_MAX", 
"LININTERP_SCATTERV_BFS_BINOMIAL_MIN",
"LININTERP_SCATTERV_BFS_BINOMIAL_MAX", 
"LININTERP_SCATTERV_UCS_BINOMIAL_MIN",
"LININTERP_SCATTERV_UCS_BINOMIAL_MAX",
"LININTERP_SCATTERV_TRAFF",
"PLOGP_SCATTERV_DFS_BINOMIAL_MIN",
"PLOGP_SCATTERV_DFS_BINOMIAL_MAX",
"PLOGP_SCATTERV_BFS_BINOMIAL_MIN",
"PLOGP_SCATTERV_BFS_BINOMIAL_MAX",
"PLOGP_SCATTERV_UCS_BINOMIAL_MIN",
"PLOGP_SCATTERV_UCS_BINOMIAL_MAX",
"PLOGP_SCATTERV_TRAFF",
"LOGGP_SCATTERV_DFS_BINOMIAL_MIN",
"LOGGP_SCATTERV_DFS_BINOMIAL_MAX",
"LOGGP_SCATTERV_BFS_BINOMIAL_MIN",
"LOGGP_SCATTERV_BFS_BINOMIAL_MAX",
"LOGGP_SCATTERV_UCS_BINOMIAL_MIN",
"LOGGP_SCATTERV_UCS_BINOMIAL_MAX",
"LOGGP_SCATTERV_TRAFF",
//gatherv operations
"MPIB_GATHERV_LINEAR",
"MPIB_GATHERV_BINOMIAL",
"MPIB_GATHERV_TRAFF",
"HOCKNEY_GATHERV_DFS_BINOMIAL_MIN", 
"HOCKNEY_GATHERV_DFS_BINOMIAL_MAX", 
"HOCKNEY_GATHERV_BFS_BINOMIAL_MIN",
"HOCKNEY_GATHERV_BFS_BINOMIAL_MAX", 
"HOCKNEY_GATHERV_UCS_BINOMIAL_MIN",
"HOCKNEY_GATHERV_UCS_BINOMIAL_MAX",
"HOCKNEY_GATHERV_TRAFF",
"LININTERP_GATHERV_DFS_BINOMIAL_MIN", 
"LININTERP_GATHERV_DFS_BINOMIAL_MAX", 
"LININTERP_GATHERV_BFS_BINOMIAL_MIN",
"LININTERP_GATHERV_BFS_BINOMIAL_MAX", 
"LININTERP_GATHERV_UCS_BINOMIAL_MIN",
"LININTERP_GATHERV_UCS_BINOMIAL_MAX",
"LININTERP_GATHERV_TRAFF",
"PLOGP_GATHERV_DFS_BINOMIAL_MIN",
"PLOGP_GATHERV_DFS_BINOMIAL_MAX",
"PLOGP_GATHERV_BFS_BINOMIAL_MIN",
"PLOGP_GATHERV_BFS_BINOMIAL_MAX",
"PLOGP_GATHERV_UCS_BINOMIAL_MIN",
"PLOGP_GATHERV_UCS_BINOMIAL_MAX",
"PLOGP_GATHERV_TRAFF",
"LOGGP_GATHERV_DFS_BINOMIAL_MIN",
"LOGGP_GATHERV_DFS_BINOMIAL_MAX",
"LOGGP_GATHERV_BFS_BINOMIAL_MIN",
"LOGGP_GATHERV_BFS_BINOMIAL_MAX",
"LOGGP_GATHERV_UCS_BINOMIAL_MIN",
"LOGGP_GATHERV_UCS_BINOMIAL_MAX",
"LOGGP_GATHERV_TRAFF",
"NUMBER_OF_OPS"
};

char model_name[256];

int collectives_initialize(MPI_Comm comm, char* subopts) {
	int rank;
	MPI_Comm_rank(comm, &rank);
	char file[256];
	char* tokens[] = {"verbose", "sgv", "model", "file"};
	char* value;
	while (*subopts != '\0') {
		switch (getsubopt(&subopts, tokens, &value)) {
		case 0:
			mpib_coll_verbose = 1;
			break;
		case 1:
			mpib_coll_sgv = atoi(value);
			break;
		case 2:
			strcpy(model_name, value);
			break;
		case 3:
			strcpy(file, value);
			break;
		default:
			if (rank == 0)
				fprintf(stderr, "Unknown suboption %s\n", value);
			break;
		}
	}
	//A model file has to be read independent of the model used. The pointer is only used by root
	FILE* stream = NULL; 
	if (rank == 0) {
		stream = fopen(file, "r");
		if (stream == NULL){
			fprintf(stderr,"The model file is not specified or does not exist\n");
			return CPM_ERR_MODEL;
		}
	}

	if (!strcmp(model_name, "Hockney")) {
		Hockney_model* model = NULL;
		if (rank == 0) {
			Hockney_read(stream, &model);
		}
		Hockney_initialize(comm, model);
		if (rank == 0)
			Hockney_free(model);
	}
	if (!strcmp(model_name, "LinInterp")) {
		LinInterp_model* model = NULL;
		if (rank == 0) {
			LinInterp_read(stream, &model);
		}
		LinInterp_initialize(comm, model);
		if (rank == 0)
			LinInterp_free(model);
	}
	if (!strcmp(model_name, "PLogP") || !strcmp(model_name, "LogGP")) {
		PLogP_model* model = NULL;
		if (rank == 0) {
			PLogP_read(stream, &model);
		}
		PLogP_initialize(comm, model);
		if (rank == 0)
			PLogP_free(model);
	}
#ifdef HAVE_LIBR
	if (!strcmp(model_name, "LMO")) {
		LMO_model* model = NULL;
		if (rank == 0) {
			LMO_read(stream, &model);
		}
		LMO_initialize(comm, model);
		//Close the model file, no matter which model it stands for
		if (rank == 0)
			LMO_free(model);
	}
#endif
	if (rank == 0){
		fclose(stream);
	}
	return 0;
}

int collectives_finalize(MPI_Comm comm) {
	if (!strcmp(model_name, "Hockney"))
		Hockney_finalize(comm);
	if (!strcmp(model_name, "LinInterp"))
		LinInterp_finalize(comm);
	if (!strcmp(model_name, "PLogP") || !strcmp(model_name, "LogGP"))
		PLogP_finalize(comm);
#ifdef HAVE_LIBR
	if (!strcmp(model_name, "LMO"))
		LMO_finalize(comm);
#endif
	return 0;
}

