#ifndef LMO_COLLECTIVES_H_
#define LMO_COLLECTIVES_H_

#ifdef HAVE_CONFIG_H
#include "models/lmo.h"
#else
#include "lmo.h"
#endif

/*!
 * \defgroup lmo_collectives LMO-based collective operations
 * \{
 */
#ifdef __cplusplus
extern "C" {
#endif

/*!
 * Initializes the instances of the LMO model on all processes in the communicatior
 * \param comm MPI communicator 
 * \param model LMO model (significant only at root)
 */
int LMO_initialize(MPI_Comm comm, LMO_model* model);

/*! Destroys the instances of the LMO model on all processes in the communicatior */
int LMO_finalize(MPI_Comm comm);

/*! Split flat-tree MPI_Scatter */
int LMO_Scatter_split_flat(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype, 
	int root, MPI_Comm comm);

/*! Split flat-tree MPI_Gather */
int LMO_Gather_split_flat(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

#ifdef __cplusplus
}
#endif
/*!
 * \}
 */

#endif /*LMO_COLLECTIVES_H_*/
