#include "config.h"
#include "loggp_collectives.h"
#include "cpm_collectives.h"

extern PLogP_model* PLogP_model_instance;

int LogGP_Scatterv_sorted_flat_asc(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatterv_sorted_flat(&PLogP_model_instance->LogGP_predictor, ASC,
		sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Gatherv_sorted_flat_asc(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gatherv_sorted_flat(&PLogP_model_instance->LogGP_predictor, ASC,
		sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
}

int LogGP_Scatterv_sorted_flat_dsc(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatterv_sorted_flat(&PLogP_model_instance->LogGP_predictor, DESC,
		sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Gatherv_sorted_flat_dsc(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gatherv_sorted_flat(&PLogP_model_instance->LogGP_predictor, DESC,
		sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
}


int LogGP_Bcast_dfs_binomial_min(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm) {
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Bcast_dfs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		buffer, count, datatype, root, comm);
}

int LogGP_Bcast_dfs_binomial_max(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm) {
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Bcast_dfs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		buffer, count, datatype, root, comm);
}

int LogGP_Bcast_bfs_binomial_min(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm) {
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Bcast_bfs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		buffer, count, datatype, root, comm);
}

int LogGP_Bcast_bfs_binomial_max(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm) {
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Bcast_bfs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		buffer, count, datatype, root, comm);
}

int LogGP_Reduce_dfs_binomial_min(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
	MPI_Op op, int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Reduce_dfs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, recvbuf, count, datatype,
		op, root, comm);
}

int LogGP_Reduce_bfs_binomial_min(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
	MPI_Op op, int root, MPI_Comm comm) {
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Reduce_bfs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, recvbuf, count, datatype,
		op, root, comm);
}

int LogGP_Reduce_bfs_binomial_max(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
	MPI_Op op, int root, MPI_Comm comm) {
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Reduce_bfs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, recvbuf, count, datatype,
		op, root, comm);
}

int LogGP_Reduce_dfs_binomial_max(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
	MPI_Op op, int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Reduce_dfs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, recvbuf, count, datatype,
		op, root, comm);
}

int LogGP_Bcast_ucs_binomial_min(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm) {
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Bcast_ucs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		buffer, count, datatype, root, comm);
}

int LogGP_Reduce_ucs_binomial_min(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
	MPI_Op op, int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Reduce_ucs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, recvbuf, count, datatype,
		op, root, comm);
}

int LogGP_Bcast_ucs_binomial_max(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm) {
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Bcast_ucs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		buffer, count, datatype, root, comm);
}

int LogGP_Reduce_ucs_binomial_max(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
	MPI_Op op, int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Reduce_ucs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, recvbuf, count, datatype,
		op, root, comm);
}

int LogGP_Scatter_dfs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm) {
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatter_dfs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

/*! DFS binomial gather based on the LogGP model. */
int LogGP_Gather_dfs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm) {
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gather_dfs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);

}

/*! DFS binomial scatter based on the LogGP model. */
int LogGP_Scatter_dfs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm) {
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatter_dfs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
}


/*! DFS binomial gather based on the LogGP model. */
int LogGP_Gather_dfs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm) {
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gather_dfs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Scatter_ucs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatter_ucs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Gather_ucs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gather_ucs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Scatter_ucs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatter_ucs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Gather_ucs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gather_ucs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Scatter_bfs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatter_bfs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Gather_bfs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gather_bfs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Scatter_bfs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatter_bfs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Gather_bfs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gather_bfs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Scatterv_dfs_binomial_min(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatterv_dfs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Gatherv_dfs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gatherv_dfs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
}

int LogGP_Scatterv_dfs_binomial_max(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatterv_dfs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Gatherv_dfs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gatherv_dfs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
}

int LogGP_Scatterv_bfs_binomial_min(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatterv_bfs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Gatherv_bfs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gatherv_bfs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
}

int LogGP_Scatterv_bfs_binomial_max(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatterv_bfs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Gatherv_bfs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gatherv_bfs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
}

int LogGP_Scatterv_ucs_binomial_min(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatterv_ucs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Gatherv_ucs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gatherv_ucs_binomial(&PLogP_model_instance->LogGP_predictor, MIN,
		sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
}

int LogGP_Scatterv_ucs_binomial_max(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatterv_ucs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
}

int LogGP_Gatherv_ucs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gatherv_ucs_binomial(&PLogP_model_instance->LogGP_predictor, MAX,
		sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
}

int LogGP_Scatterv_Traff(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Scatterv_Traff(&PLogP_model_instance->LogGP_predictor,
		sendbuf, sendcounts, displs, sendtype,
		recvbuf, recvcount, recvtype,
		root, comm);
}

int LogGP_Gatherv_Traff(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	if (PLogP_model_instance == NULL)
		return CPM_ERR_MODEL;
	return CPM_Gatherv_Traff(&PLogP_model_instance->LogGP_predictor,
		sendbuf, sendcount, sendtype,
		recvbuf, recvcounts, displs, recvtype,
		root, comm);
}
