#ifndef PLOGP_COLLECTIVES_H_
#define PLOGP_COLLECTIVES_H_

#ifdef HAVE_CONFIG_H
#include "models/plogp.h"
#else
#include "plogp.h"
#endif

/*!
 * \defgroup plogp_collectives PLogP-based collective operations
 * \{
 */
#ifdef __cplusplus
extern "C" {
#endif

/*!
 * Initializes the instances of the PLogP model at all processes in the communicatior.
 * \param comm MPI communicator
 * \param model PLogP model (significant only at root)
 */
int PLogP_initialize(MPI_Comm comm, PLogP_model* model);

/*! Destroys the instances of the PLogP model at all processes in the communicatior.
 */
int PLogP_finalize(MPI_Comm comm);

/*! Sorted flat-tree scatterv based on the PLogP model. */
int PLogP_Scatterv_sorted_flat_asc(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! Sorted flat-tree gatherv based on the PLogP model. */
int PLogP_Gatherv_sorted_flat_asc(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! Sorted flat-tree scatterv based on the PLogP model. */
int PLogP_Scatterv_sorted_flat_dsc(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! Sorted flat-tree gatherv based on the PLogP model. */
int PLogP_Gatherv_sorted_flat_dsc(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! DFS binomial bcast based on the PLogP model. */
int PLogP_Bcast_dfs_binomial_min(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);

/*! DFS binomial reduce based on the PLogP model. */
int PLogP_Reduce_dfs_binomial_min(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
	MPI_Op op, int root, MPI_Comm comm);

/*! DFS binomial bcast based on the PLogP model. */
int PLogP_Bcast_dfs_binomial_max(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);

/*! BFS binomial reduce based on the PLogP model. */
int PLogP_Reduce_bfs_binomial_min(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
	MPI_Op op, int root, MPI_Comm comm);

/*! BFS binomial reduce based on the PLogP model. */
int PLogP_Reduce_bfs_binomial_max(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
	MPI_Op op, int root, MPI_Comm comm);

/*! BFS binomial bcast based on the PLogP model. */
int PLogP_Bcast_bfs_binomial_min(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);

/*! BFS binomial bcast based on the PLogP model. */
int PLogP_Bcast_bfs_binomial_max(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);

/*! DFS binomial reduce based on the PLogP model. */
int PLogP_Reduce_dfs_binomial_max(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
	MPI_Op op, int root, MPI_Comm comm);

/*! UCS binomial bcast based on the PLogP model. */
int PLogP_Bcast_ucs_binomial_min(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);

/*! UCS binomial reduce based on the PLogP model. */
int PLogP_Reduce_ucs_binomial_min(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
	MPI_Op op, int root, MPI_Comm comm);

/*! UCS binomial bcast based on the PLogP model. */
int PLogP_Bcast_ucs_binomial_max(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);

/*! UCS binomial reduce based on the PLogP model. */
int PLogP_Reduce_ucs_binomial_max(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
	MPI_Op op, int root, MPI_Comm comm);

/*! DFS binomial scatter based on the PLogP model. */
int PLogP_Scatter_dfs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! DFS binomial gather based on the PLogP model. */
int PLogP_Gather_dfs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! DFS binomial scatter based on the PLogP model. */
int PLogP_Scatter_dfs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! DFS binomial gather based on the PLogP model. */
int PLogP_Gather_dfs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! UCS binomial scatter based on the PLogP model. */
int PLogP_Scatter_ucs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! UCS binomial gather based on the PLogP model. */
int PLogP_Gather_ucs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! UCS binomial scatter based on the PLogP model. */
int PLogP_Scatter_ucs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! UCS binomial gather based on the PLogP model. */
int PLogP_Gather_ucs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! BFS binomial scatter based on the PLogP model. */
int PLogP_Scatter_bfs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! BFS binomial gather based on the PLogP model. */
int PLogP_Gather_bfs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! BFS binomial scatter based on the PLogP model. */
int PLogP_Scatter_bfs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! BFS binomial gather based on the PLogP model. */
int PLogP_Gather_bfs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! DFS binomial scatterv based on the PLogP model. */
int PLogP_Scatterv_dfs_binomial_min(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! DFS binomial gatherv based on the PLogP model. */
int PLogP_Gatherv_dfs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! DFS binomial scatterv based on the PLogP model. */
int PLogP_Scatterv_dfs_binomial_max(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! DFS binomial gatherv based on the PLogP model. */
int PLogP_Gatherv_dfs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! BFS binomial scatterv based on the PLogP model. */
int PLogP_Scatterv_bfs_binomial_min(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! BFS binomial gatherv based on the PLogP model. */
int PLogP_Gatherv_bfs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! BFS binomial scatterv based on the PLogP model. */
int PLogP_Scatterv_bfs_binomial_max(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! BFS binomial gatherv based on the PLogP model. */
int PLogP_Gatherv_bfs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! UCS binomial scatterv based on the PLogP model. */
int PLogP_Scatterv_ucs_binomial_min(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! UCS binomial gatherv based on the PLogP model. */
int PLogP_Gatherv_ucs_binomial_min(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! UCS binomial scatterv based on the PLogP model. */
int PLogP_Scatterv_ucs_binomial_max(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! UCS binomial gatherv based on the PLogP model. */
int PLogP_Gatherv_ucs_binomial_max(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! Scatterv based on modified Traff using the PLogP model. */
int PLogP_Scatterv_Traff(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

/*! Gatherv based on modified Traff using the PLogP model. */
int PLogP_Gatherv_Traff(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype,
	int root, MPI_Comm comm);


#ifdef __cplusplus
}
#endif
/*!
 * \}
 */

#endif /*PLOGP_COLLECTIVES_H_*/

