#include "cpm.h"
#include "MPIBlib/benchmarks/mpib.h"

void CPM_print_model(const char* model, MPIB_msgset msgset, int parallel, MPIB_precision precision, double time) {
	printf("#model\t\t%s\n", model);
	MPIB_print_msgset(msgset);
	MPIB_print_p2p(parallel);
	MPIB_print_precision(precision);
	printf("#building time\t%le\n", time);
}

void CPM_print_p2pp_results(int n, int M, MPIB_precision precision, MPIB_result* p2pp, MPIB_result* p2p_0, MPIB_result* p2p_M)
{
	printf("#msg_size\t%d\n", M);
	MPIB_print_precision(precision);
	printf("#p2pp\t\ttime\t\twtick\t\treps\tci\t\ttime\t\ttime\n");
	int i, index;
	for(i = 0, index = 0; i < n - 1; i++)
	{
		int j;
		for(j = i + 1; j < n; j++)
		{
			int k;
			for(k = j + 1; k < n; k++, index++)
			{
				int x = 3 * index;
				double T[2];
				T[0] = (p2p_0[MPIB_IJ2INDEX(n, i, j)].T + p2p_M[MPIB_IJ2INDEX(n, i, j)].T) / 2;
				T[1] = (p2p_0[MPIB_IJ2INDEX(n, i, k)].T + p2p_M[MPIB_IJ2INDEX(n, i, k)].T) / 2;
				printf("%d-%d-%d\t\t%le\t%le\t%s\t%d\t%le\t%le\t%le\t%s\n", i, j, k,
					p2pp[x].T, p2pp[x].wtick, p2pp[x].wtick < p2pp[x].T ? "" : "*",
					p2pp[x].reps, p2pp[x].ci,
					T[0], T[1],
					T[0] < p2pp[x].T && T[1] < p2pp[x].T ? "" : "*");
				x++;
				T[0] = (p2p_0[MPIB_IJ2INDEX(n, i, j)].T + p2p_M[MPIB_IJ2INDEX(n, i, j)].T) / 2;
				T[1] = (p2p_0[MPIB_IJ2INDEX(n, j, k)].T + p2p_M[MPIB_IJ2INDEX(n, j, k)].T) / 2;
				printf("%d-%d-%d\t\t%le\t%le\t%s\t%d\t%le\t%le\t%le\t%s\n", j, i, k,
					p2pp[x].T, p2pp[x].wtick, p2pp[x].wtick < p2pp[x].T ? "" : "*",
					p2pp[x].reps, p2pp[x].ci,
					T[0], T[1],
					T[0] < p2pp[x].T && T[1] < p2pp[x].T ? "" : "*");
				x++;
				T[0] = (p2p_0[MPIB_IJ2INDEX(n, i, k)].T + p2p_M[MPIB_IJ2INDEX(n, i, k)].T) / 2;
				T[1] = (p2p_0[MPIB_IJ2INDEX(n, j, k)].T + p2p_M[MPIB_IJ2INDEX(n, j, k)].T) / 2;
				printf("%d-%d-%d\t\t%le\t%le\t%s\t%d\t%le\t%le\t%le\t%s\n", k, i, j,
					p2pp[x].T, p2pp[x].wtick, p2pp[x].wtick < p2pp[x].T ? "" : "*",
					p2pp[x].reps, p2pp[x].ci,
					T[0], T[1],
					T[0] < p2pp[x].T && T[1] < p2pp[x].T ? "" : "*");
			}
		}
	}
}

void CPM_print_p2pp_results_(int n, int M, MPIB_precision precision, MPIB_result* p2pp, MPIB_result* p2p_0[2], MPIB_result* p2p_M[2])
{
	printf("#msg_size\t%d\n", M);
	MPIB_print_precision(precision);
	printf("#p2pp\t\ttime\t\twtick\t\treps\tci\t\ttime\t\ttime\n");
	int i, index;
	for(i = 0, index = 0; i < n - 1; i++)
	{
		int j;
		for(j = i + 1; j < n; j++)
		{
			int k;
			for(k = j + 1; k < n; k++, index++)
			{
				int x = 3 * index;
				double T[2];
				T[0] = (p2p_0[0][x].T + p2p_M[0][x].T) / 2;
				T[1] = (p2p_0[1][x].T + p2p_M[1][x].T) / 2;
				printf("%d-%d-%d\t\t%le\t%le\t%s\t%d\t%le\t%le\t%le\t%s\n", i, j, k,
					p2pp[x].T, p2pp[x].wtick, p2pp[x].wtick < p2pp[x].T ? "" : "*",
					p2pp[x].reps, p2pp[x].ci,
					T[0], T[1],
					T[0] < p2pp[x].T && T[1] < p2pp[x].T ? "" : "*");
				x++;
				T[0] = (p2p_0[0][x].T + p2p_M[0][x].T) / 2;
				T[1] = (p2p_0[1][x].T + p2p_M[1][x].T) / 2;
				printf("%d-%d-%d\t\t%le\t%le\t%s\t%d\t%le\t%le\t%le\t%s\n", j, i, k,
					p2pp[x].T, p2pp[x].wtick, p2pp[x].wtick < p2pp[x].T ? "" : "*",
					p2pp[x].reps, p2pp[x].ci,
					T[0], T[1],
					T[0] < p2pp[x].T && T[1] < p2pp[x].T ? "" : "*");
				x++;
				T[0] = (p2p_0[0][x].T + p2p_M[0][x].T) / 2;
				T[1] = (p2p_0[1][x].T + p2p_M[1][x].T) / 2;
				printf("%d-%d-%d\t\t%le\t%le\t%s\t%d\t%le\t%le\t%le\t%s\n", k, i, j,
					p2pp[x].T, p2pp[x].wtick, p2pp[x].wtick < p2pp[x].T ? "" : "*",
					p2pp[x].reps, p2pp[x].ci,
					T[0], T[1],
					T[0] < p2pp[x].T && T[1] < p2pp[x].T ? "" : "*");
			}
		}
	}
}
