#include "cpm_predict_collectives.h"
#include "lininterp.h"
#include "hockney.h"
#include "plogp.h"
#include "lmo.h"
#include "cpm_models.h"
#include "cpm_predict_trees.hpp"
#include "cpm_brsg_tree_builders.hpp"
#include "cpm_sgv_tree_builders.hpp"
#include "MPIBlib/collectives/brsg_tree_builders.hpp"
#include "MPIBlib/collectives/sgv_tree_builders.hpp"

extern "C" double CPM_predict_brsg(CPM_predictor* predictor, int size, int root, int size_bytes, CPM_coll_ops operation) {
	double result;
	switch (operation) {
	case MPIB_SCATTER_LINEAR:
	case MPIB_GATHER_LINEAR:
		//result = CPM_predict_flat_sg_parallel(predictor, size, root, size_bytes);
		result = CPM_predict_flat_sg_serial(predictor, size, root, size_bytes);
		break;
	case MPIB_SCATTER_BINOMIAL:
	case MPIB_GATHER_BINOMIAL:
		result = CPM_predict_tree_brsg(MPIB::BRSG::Binomial_builder(), L2R, size, root, size_bytes, predictor, SG);
		break;
	case MPIB_BCAST_BINOMIAL: 
	case MPIB_REDUCE_BINOMIAL:
		result = CPM_predict_tree_brsg(MPIB::BRSG::Binomial_builder(), L2R, size, root, size_bytes, predictor, BR);
		break;
	/*
	 * The direction of children traversal in CPM_predict for the best-case prediction only depends
	 * on the tree form - starting from the smaller subtrees  and ending with the larger subtrees.
	 * It does not depend on the operation, since the operation follows different direction depending
	 * on top-down or bottom-up propagation
	 */
	case HOCKNEY_SCATTER_DFS_BINOMIAL_MAX:
	case HOCKNEY_GATHER_DFS_BINOMIAL_MAX:
	case LININTERP_SCATTER_DFS_BINOMIAL_MAX:
	case LININTERP_GATHER_DFS_BINOMIAL_MAX:
	case PLOGP_SCATTER_DFS_BINOMIAL_MAX:
	case PLOGP_GATHER_DFS_BINOMIAL_MAX:
	case LOGGP_SCATTER_DFS_BINOMIAL_MAX:
	case LOGGP_GATHER_DFS_BINOMIAL_MAX:
		result = CPM_predict_tree_brsg(CPM::BRSG::DFS_binomial_builder(predictor, MAX, SG), L2R, size, root, size_bytes, predictor, SG);
		break;
	case HOCKNEY_BCAST_DFS_BINOMIAL_MAX:
	case HOCKNEY_REDUCE_DFS_BINOMIAL_MAX:
	case LININTERP_BCAST_DFS_BINOMIAL_MAX:
	case LININTERP_REDUCE_DFS_BINOMIAL_MAX:
	case PLOGP_BCAST_DFS_BINOMIAL_MAX:
	case PLOGP_REDUCE_DFS_BINOMIAL_MAX:
	case LOGGP_BCAST_DFS_BINOMIAL_MAX:
	case LOGGP_REDUCE_DFS_BINOMIAL_MAX:
		result = CPM_predict_tree_brsg(CPM::BRSG::DFS_binomial_builder(predictor, MAX, BR), L2R, size, root, size_bytes, predictor, BR);
		break;
	case HOCKNEY_SCATTER_DFS_BINOMIAL_MIN:
	case HOCKNEY_GATHER_DFS_BINOMIAL_MIN:
	case LININTERP_SCATTER_DFS_BINOMIAL_MIN:
	case LININTERP_GATHER_DFS_BINOMIAL_MIN:
	case PLOGP_SCATTER_DFS_BINOMIAL_MIN:
	case PLOGP_GATHER_DFS_BINOMIAL_MIN:
	case LOGGP_SCATTER_DFS_BINOMIAL_MIN:
	case LOGGP_GATHER_DFS_BINOMIAL_MIN:
		result = CPM_predict_tree_brsg(CPM::BRSG::DFS_binomial_builder(predictor, MIN, SG), L2R, size, root, size_bytes, predictor, SG);
		break;
	case HOCKNEY_BCAST_DFS_BINOMIAL_MIN: 
	case HOCKNEY_REDUCE_DFS_BINOMIAL_MIN: 
	case LININTERP_BCAST_DFS_BINOMIAL_MIN: 
	case LININTERP_REDUCE_DFS_BINOMIAL_MIN: 
	case PLOGP_BCAST_DFS_BINOMIAL_MIN:
	case PLOGP_REDUCE_DFS_BINOMIAL_MIN:
	case LOGGP_BCAST_DFS_BINOMIAL_MIN:
	case LOGGP_REDUCE_DFS_BINOMIAL_MIN:
		result = CPM_predict_tree_brsg(CPM::BRSG::DFS_binomial_builder(predictor, MIN, BR), L2R, size, root, size_bytes, predictor, BR);
		break;
	case HOCKNEY_SCATTER_BFS_BINOMIAL_MAX:
	case HOCKNEY_GATHER_BFS_BINOMIAL_MAX:
	case LININTERP_SCATTER_BFS_BINOMIAL_MAX:
	case LININTERP_GATHER_BFS_BINOMIAL_MAX:
	case PLOGP_SCATTER_BFS_BINOMIAL_MAX:
	case PLOGP_GATHER_BFS_BINOMIAL_MAX:
	case LOGGP_SCATTER_BFS_BINOMIAL_MAX:
	case LOGGP_GATHER_BFS_BINOMIAL_MAX:
		result = CPM_predict_tree_brsg(CPM::BRSG::BFS_binomial_builder(predictor, MAX, SG), R2L, size, root, size_bytes, predictor, SG);
		break;
	case HOCKNEY_BCAST_BFS_BINOMIAL_MAX:
	case HOCKNEY_REDUCE_BFS_BINOMIAL_MAX:
	case LININTERP_BCAST_BFS_BINOMIAL_MAX:
	case LININTERP_REDUCE_BFS_BINOMIAL_MAX:
	case PLOGP_BCAST_BFS_BINOMIAL_MAX:
	case PLOGP_REDUCE_BFS_BINOMIAL_MAX:
	case LOGGP_BCAST_BFS_BINOMIAL_MAX:
	case LOGGP_REDUCE_BFS_BINOMIAL_MAX:
		result = CPM_predict_tree_brsg(CPM::BRSG::BFS_binomial_builder(predictor, MAX, BR), R2L, size, root, size_bytes, predictor, BR);
		break;
	case HOCKNEY_SCATTER_BFS_BINOMIAL_MIN:
	case HOCKNEY_GATHER_BFS_BINOMIAL_MIN:
	case LININTERP_SCATTER_BFS_BINOMIAL_MIN:
	case LININTERP_GATHER_BFS_BINOMIAL_MIN:
	case PLOGP_SCATTER_BFS_BINOMIAL_MIN:
	case PLOGP_GATHER_BFS_BINOMIAL_MIN:
	case LOGGP_SCATTER_BFS_BINOMIAL_MIN:
	case LOGGP_GATHER_BFS_BINOMIAL_MIN:
		result = CPM_predict_tree_brsg(CPM::BRSG::BFS_binomial_builder(predictor, MIN, SG), R2L, size, root, size_bytes, predictor, SG);
		break;
	case HOCKNEY_BCAST_BFS_BINOMIAL_MIN:
	case HOCKNEY_REDUCE_BFS_BINOMIAL_MIN:
	case LININTERP_BCAST_BFS_BINOMIAL_MIN:
	case LININTERP_REDUCE_BFS_BINOMIAL_MIN:
	case PLOGP_BCAST_BFS_BINOMIAL_MIN:
	case PLOGP_REDUCE_BFS_BINOMIAL_MIN:
	case LOGGP_BCAST_BFS_BINOMIAL_MIN:
	case LOGGP_REDUCE_BFS_BINOMIAL_MIN:
		result = CPM_predict_tree_brsg(CPM::BRSG::BFS_binomial_builder(predictor, MIN, BR), R2L, size, root, size_bytes, predictor, BR);
		break;
	case HOCKNEY_SCATTER_UCS_BINOMIAL_MAX:
	case HOCKNEY_GATHER_UCS_BINOMIAL_MAX:
	case LININTERP_SCATTER_UCS_BINOMIAL_MAX:
	case LININTERP_GATHER_UCS_BINOMIAL_MAX:
	case PLOGP_SCATTER_UCS_BINOMIAL_MAX:
	case PLOGP_GATHER_UCS_BINOMIAL_MAX:
	case LOGGP_SCATTER_UCS_BINOMIAL_MAX:
	case LOGGP_GATHER_UCS_BINOMIAL_MAX:
		result = CPM_predict_tree_brsg(CPM::BRSG::UCS_binomial_builder(predictor, MAX, SG), R2L, size, root, size_bytes, predictor, SG);
		break;
	case HOCKNEY_BCAST_UCS_BINOMIAL_MAX:
	case HOCKNEY_REDUCE_UCS_BINOMIAL_MAX:
	case LININTERP_BCAST_UCS_BINOMIAL_MAX:
	case LININTERP_REDUCE_UCS_BINOMIAL_MAX:
	case PLOGP_BCAST_UCS_BINOMIAL_MAX:
	case PLOGP_REDUCE_UCS_BINOMIAL_MAX:
	case LOGGP_BCAST_UCS_BINOMIAL_MAX:
	case LOGGP_REDUCE_UCS_BINOMIAL_MAX:
		result = CPM_predict_tree_brsg(CPM::BRSG::UCS_binomial_builder(predictor, MAX, BR), R2L, size, root, size_bytes, predictor, BR);
		break;
	case HOCKNEY_SCATTER_UCS_BINOMIAL_MIN:
	case HOCKNEY_GATHER_UCS_BINOMIAL_MIN:
	case LININTERP_SCATTER_UCS_BINOMIAL_MIN:
	case LININTERP_GATHER_UCS_BINOMIAL_MIN:
	case PLOGP_SCATTER_UCS_BINOMIAL_MIN:
	case PLOGP_GATHER_UCS_BINOMIAL_MIN:
	case LOGGP_SCATTER_UCS_BINOMIAL_MIN:
	case LOGGP_GATHER_UCS_BINOMIAL_MIN:
		result = CPM_predict_tree_brsg(CPM::BRSG::UCS_binomial_builder(predictor, MIN, SG), R2L, size, root, size_bytes, predictor, SG);
		break;
	case HOCKNEY_BCAST_UCS_BINOMIAL_MIN:
	case HOCKNEY_REDUCE_UCS_BINOMIAL_MIN:
	case LININTERP_BCAST_UCS_BINOMIAL_MIN:
	case LININTERP_REDUCE_UCS_BINOMIAL_MIN:
	case PLOGP_BCAST_UCS_BINOMIAL_MIN:
	case PLOGP_REDUCE_UCS_BINOMIAL_MIN:
	case LOGGP_BCAST_UCS_BINOMIAL_MIN:
	case LOGGP_REDUCE_UCS_BINOMIAL_MIN:
		result = CPM_predict_tree_brsg(CPM::BRSG::UCS_binomial_builder(predictor, MIN, BR), R2L, size, root, size_bytes, predictor, BR);
		break;
	default:
		fprintf(stderr, "Did not find a match for operation\n");
		exit(-1);
	}
	return result;
}

extern "C" double CPM_predict_sgv(CPM_predictor* predictor, int size, int root, int* size_bytes, CPM_coll_ops operation) {
	double result;
	switch (operation) {
	case MPIB_SCATTERV_LINEAR:
	case MPIB_GATHERV_LINEAR:
		result = CPM_predict_flat_sgv_parallel(predictor, size, root, size_bytes);
		break;
	case MPIB_SCATTERV_BINOMIAL:
	case MPIB_GATHERV_BINOMIAL:
		result = CPM_predict_tree_sgv(MPIB::SGv::Binomial_builder(), L2R, size, root, size_bytes, predictor, SGV);
		break;
	case MPIB_SCATTERV_TRAFF:
	case MPIB_GATHERV_TRAFF:
		result = CPM_predict_tree_sgv(MPIB::SGv::Traff_builder(), L2R, size, root, size_bytes,predictor, SGV);
		break;
	/*
	 * The direction of children traversal in CPM_predict for the best-case prediction only depends 
	 * on the tree form - starting from the smaller subtrees  and ending with the larger subtrees.  
	 * It does not depend on the operation, since the operation follows different direction depending 
	 * on top-down or bottom-up propagation
	 */
	case HOCKNEY_SCATTERV_DFS_BINOMIAL_MIN:
	case HOCKNEY_GATHERV_DFS_BINOMIAL_MIN:
	case LININTERP_SCATTERV_DFS_BINOMIAL_MIN:
	case LININTERP_GATHERV_DFS_BINOMIAL_MIN:
	case PLOGP_SCATTERV_DFS_BINOMIAL_MIN:
	case PLOGP_GATHERV_DFS_BINOMIAL_MIN:
	case LOGGP_SCATTERV_DFS_BINOMIAL_MIN:
	case LOGGP_GATHERV_DFS_BINOMIAL_MIN:

		result = CPM_predict_tree_sgv(CPM::SGv::DFS_binomial_builder(predictor, MIN), L2R, size, root, size_bytes, predictor, SGV);
		break;
	case HOCKNEY_SCATTERV_BFS_BINOMIAL_MIN:
	case HOCKNEY_GATHERV_BFS_BINOMIAL_MIN:
	case LININTERP_SCATTERV_BFS_BINOMIAL_MIN:
	case LININTERP_GATHERV_BFS_BINOMIAL_MIN:
	case PLOGP_SCATTERV_BFS_BINOMIAL_MIN:
	case PLOGP_GATHERV_BFS_BINOMIAL_MIN:
	case LOGGP_SCATTERV_BFS_BINOMIAL_MIN:
	case LOGGP_GATHERV_BFS_BINOMIAL_MIN:
		result = CPM_predict_tree_sgv(CPM::SGv::BFS_binomial_builder(predictor, MIN), R2L, size, root, size_bytes, predictor, SGV);
		break;
	case HOCKNEY_SCATTERV_UCS_BINOMIAL_MIN:
	case HOCKNEY_GATHERV_UCS_BINOMIAL_MIN:
	case LININTERP_SCATTERV_UCS_BINOMIAL_MIN:
	case LININTERP_GATHERV_UCS_BINOMIAL_MIN:
	case PLOGP_SCATTERV_UCS_BINOMIAL_MIN:
	case PLOGP_GATHERV_UCS_BINOMIAL_MIN:
	case LOGGP_SCATTERV_UCS_BINOMIAL_MIN:
	case LOGGP_GATHERV_UCS_BINOMIAL_MIN:
		result = CPM_predict_tree_sgv(CPM::SGv::UCS_binomial_builder(predictor, MIN), R2L, size, root, size_bytes, predictor, SGV);
		break;
	case HOCKNEY_SCATTERV_DFS_BINOMIAL_MAX:
	case HOCKNEY_GATHERV_DFS_BINOMIAL_MAX:
	case LININTERP_SCATTERV_DFS_BINOMIAL_MAX:
	case LININTERP_GATHERV_DFS_BINOMIAL_MAX:
	case PLOGP_SCATTERV_DFS_BINOMIAL_MAX:
	case PLOGP_GATHERV_DFS_BINOMIAL_MAX:
	case LOGGP_SCATTERV_DFS_BINOMIAL_MAX:
	case LOGGP_GATHERV_DFS_BINOMIAL_MAX:
		result = CPM_predict_tree_sgv(CPM::SGv::DFS_binomial_builder(predictor, MAX), L2R, size, root, size_bytes, predictor, SGV);
		break;
	case HOCKNEY_SCATTERV_BFS_BINOMIAL_MAX:
	case HOCKNEY_GATHERV_BFS_BINOMIAL_MAX:
	case LININTERP_SCATTERV_BFS_BINOMIAL_MAX:
	case LININTERP_GATHERV_BFS_BINOMIAL_MAX:
	case PLOGP_SCATTERV_BFS_BINOMIAL_MAX:
	case PLOGP_GATHERV_BFS_BINOMIAL_MAX:
	case LOGGP_SCATTERV_BFS_BINOMIAL_MAX:
	case LOGGP_GATHERV_BFS_BINOMIAL_MAX:
		result = CPM_predict_tree_sgv(CPM::SGv::BFS_binomial_builder(predictor, MAX), R2L, size, root, size_bytes, predictor, SGV);
		break;
	case HOCKNEY_SCATTERV_UCS_BINOMIAL_MAX:
	case HOCKNEY_GATHERV_UCS_BINOMIAL_MAX:
	case LININTERP_SCATTERV_UCS_BINOMIAL_MAX:
	case LININTERP_GATHERV_UCS_BINOMIAL_MAX:
	case PLOGP_SCATTERV_UCS_BINOMIAL_MAX:
	case PLOGP_GATHERV_UCS_BINOMIAL_MAX:
	case LOGGP_SCATTERV_UCS_BINOMIAL_MAX:
	case LOGGP_GATHERV_UCS_BINOMIAL_MAX:
			result = CPM_predict_tree_sgv(CPM::SGv::UCS_binomial_builder(predictor, MAX), R2L, size, root, size_bytes, predictor, SGV);
		break;
	/*
	 * Traff trees are special - we build them with potentially larger subtrees on the right, and communication and prediction is done
	   in the same direction as DFS trees
	 */
	case HOCKNEY_SCATTERV_TRAFF:
	case HOCKNEY_GATHERV_TRAFF:
	case LININTERP_SCATTERV_TRAFF:
	case LININTERP_GATHERV_TRAFF:
	case PLOGP_SCATTERV_TRAFF:
	case PLOGP_GATHERV_TRAFF:
	case LOGGP_SCATTERV_TRAFF:
	case LOGGP_GATHERV_TRAFF:
		result = CPM_predict_tree_sgv(CPM::SGv::Traff_builder(predictor), L2R, size, root, size_bytes, predictor, SGV);
		break;
	default:
		fprintf(stderr, "Did not find a match for operation\n");
		exit(-1);
	}
	return result;
}
