#ifndef CPM_PREDICT_COLLECTIVES_H_
#define CPM_PREDICT_COLLECTIVES_H_

#include "collectives/cpm_coll_ops_list.h"
#include "cpm_models.h"

/*!
 * \defgroup prediction Prediction of communication time
 * This module provides generic predict functions
 * \{
 */

#ifdef __cplusplus
extern "C" {
#endif

/*! Predicts the execution time of Bcast, Reduce, Scatter, Gather operations */
double CPM_predict_brsg(CPM_predictor* predictor, int size, int root, int size_bytes, CPM_coll_ops operation);

/*! Predicts the execution time of Scatterv, Gatherv operations */
double CPM_predict_sgv(CPM_predictor* predictor, int size, int root, int* size_bytes, CPM_coll_ops operation);

/*! Predicts the execution time of flat-tree Scatterv, Gatherv operations */
double CPM_predict_flat_sgv(CPM_predictor* predictor, int size, int root, int* size_bytes);

/*! Predicts the execution time of flat-tree Scatter, Gather operations */
double CPM_predict_flat_sg(CPM_predictor* predictor, int size, int root, int size_bytes);

/*! return maximum of all the P2P transfer times */
double CPM_predict_flat_sgv_parallel(CPM_predictor* predictor, int size, int root, int* size_bytes);

/*! return maximum of all the P2P transfer times */
double CPM_predict_flat_sg_parallel(CPM_predictor* predictor, int size, int root, int size_bytes);

/*! return sum of all the P2P transfer times */
double CPM_predict_flat_sgv_serial(CPM_predictor* predictor, int size, int root, int* size_bytes);

/*! return sum of all the P2P transfer times */
double CPM_predict_flat_sg_serial(CPM_predictor* predictor, int size, int root, int size_bytes);

#ifdef __cplusplus
}
#endif

/*!
 * \}
 */

#endif /* CPM_PREDICT_COLLECTIVES_H_ */
