#ifndef _PLOGP_P2P_MODEL_HPP
#define _PLOGP_P2P_MODEL_HPP

#include <boost/serialization/access.hpp>
#include <gsl/gsl_fit.h>

extern "C" {
#include "../../logp_mpi-1.4/logp_mpi.h"
}

/*! The PLogP P2P model */
class plogp_p2p_model {
private:
	friend class boost::serialization::access;
	logp_params * logp;

public:
	plogp_p2p_model() {logp = NULL;}
	~plogp_p2p_model() { logp_free(logp);}

	/*! Serializes the model */
	template <class Archive> void serialize(Archive & ar, const unsigned int version) {
		if (logp == NULL) {
			logp = (logp_params*)malloc(sizeof(logp_params));
			logp->descr = (char*)malloc(sizeof(char) * 4);
			strcpy(logp->descr, "CPM");
			logp->params = NULL;
		}
		ar & logp->nsizes;
		ar & logp->nsizes_max;
		ar & logp->L;
		if (logp->params == NULL) 
			logp->params = (logp_size_params*)malloc(sizeof(logp_size_params) * logp->nsizes);
		int i ;	
		logp_size_params *params;
		for (i=0,params = logp->params; i < logp->nsizes; i++,params++) {
			ar & params->size;
			ar & params->time;
			ar & params->RTT;
			ar & params->RTT_min;
			ar & params->os;
			ar & params->os_cnfint;
			ar & params->os_min;
			ar & params->_or;
			ar & params->or_cnfint;
			ar & params->or_min;
			ar & params->g;
		}
	}

	void estimate(MPI_Comm comm, MPIB_msgset msgset, MPIB_precision precision, int i, int j) {
		int rank;
		MPI_Comm_rank(comm, &rank);
		logp_params *tmp; 
		logp_measure(&tmp, (char*)"CPM", comm, i, j, 0,
			precision.min_reps, precision.max_reps, precision.cl, precision.eps,
			msgset.max_diff, (msgset.min_size > 0) ? msgset.min_size : 1, msgset.max_size, msgset.max_num,
			LOGP_MPI_SEND, LOGP_MPI_RECV, LOGP_BUF_STATIC, LOGP_OPT_GM_DIRECT, LOGP_DIAG_SILENT);
		if (rank == i)
			logp = tmp;
	}

	void write_header(ostream &os) {}
	void write_elem(int i, int j, ostream &os) {
		os << "\n\n";
		os << "#" << i <<"-" << j <<"\n" ;
		os << "# LogP network performance data: " << logp->descr << endl;
		os << "# Latency = ";
		os.setf(ios::fixed,ios::floatfield);
		os.precision(7);
		os << logp->L << endl;
		os << "# time      bytes os        os_min    os_cnfint or        or_min    or_cnfint g\n" ;
		for (i = 0; i < logp->nsizes; i++) {
			logp_size_params *lsp = &logp->params[i];
			os.width(9);
			os << lsp->time << " " ;
			os.width(7);
			os << lsp->size << " ";
			os.width(0);
			os << lsp->os << " ";
			os << lsp-> os_min << " ";
			os << lsp->os_cnfint << " " << lsp->_or << " " << lsp->or_min << " " << lsp->or_cnfint << " " << lsp->g << "\n" ;
		}
	}
	void write_footer(ostream &os) {
		os << "\n\n";
	}
};

#endif /* _PLOGP_P2P_MODEL_HPP */
