#ifndef OBSERVATION_H_
#define OBSERVATION_H_

#include "cpm_model.h"
#include <gsl/gsl_spline.h>
#include <stdio.h>
#ifdef HAVE_CONFIG_H
#include "MPIBlib/benchmarks/mpib_measurement.h"
#else
#include "mpib_measurement.h"
#endif

/*!
 * \defgroup lininterp The linear interpolation model
 * \{
 */
#ifdef __cplusplus
extern "C" {
#endif

/*! The linear interpolation model */
typedef struct LinInterp_model {
	/*! Predictor */
	CPM_predictor predictor;
	/*! Number of nodes */
	int n;
	/*! Number of points */
	int num_points;
	/*! Splines */
	gsl_spline** splines;
	/*! Accelerators */
	gsl_interp_accel** accels;
} LinInterp_model;

/*! Allocates the linear interpolation model */
LinInterp_model* LinInterp_alloc(int size, int num_points);

/*! Frees the linear interpolation model */
void LinInterp_free(LinInterp_model* model);

/*! Estimates the linear interpolation model */
void LinInterp_estimate(MPI_Comm comm, MPIB_precision precision, MPIB_msgset msgset, int parallel, LinInterp_model** _model);

/*! Reads the linear interpolation model */
void LinInterp_read(FILE* stream, LinInterp_model** model);

/*! Writes the linear interpolation model */
void LinInterp_write(FILE* stream, const LinInterp_model* model);

/*! Predicts the p2p communication execution time using the linear interpolation model */
double LinInterp_predict_p2p(void* _this, int i, int j, int M);

#ifdef __cplusplus
}
#endif
/*!
 * \}
 */

#endif /*OBSERVATION_H_*/
